import uuid
import enum


class FOLIONamespaces(enum.Enum):
    """
    ENUM for UUID v5 Namespaces. Helping the FolioUUID to create
    unique deterministic uuids within a folio tenant
    """

    # Inventory
    # uuid.uuid5(FolioUUID.base_namespace, "holdings")
    holdings = uuid.UUID("81b6a778-b024-5050-9712-81d01c2ee33b")
    # uuid.uuid5(FolioUUID.base_namespace, "items")
    items = uuid.UUID("4ccaf819-a366-55f2-9a99-4e37f8d4031b")
    # uuid.uuid5(FolioUUID.base_namespace, "instances")
    instances = uuid.UUID("09d734f7-d5df-5fa1-9b5e-31d34f4bcfef")

    # SRS
    # uuid.uuid5(FolioUUID.base_namespace, "srs_records")
    srs_records = uuid.UUID("13e60b21-777c-5cb6-b7d8-2855b5d41852")
    # uuid.uuid5(FolioUUID.base_namespace, "raw_records")
    raw_records = uuid.UUID("cd2ba9c1-bb21-5148-8a39-3190fd78a047")
    # uuid.uuid5(FolioUUID.base_namespace, "parsed_records")
    parsed_records = uuid.UUID("9857b7bc-569c-53e4-bb1f-5449f8c4f62b")

    # Circulation
    # uuid.uuid5(FolioUUID.base_namespace, "loans")
    loans = uuid.UUID("08fb857e-4aeb-5eb8-a497-68495cc894f2")
    # uuid.uuid5(FolioUUID.base_namespace, "requests")
    requests = uuid.UUID("85934592-11a7-5889-895c-ca53cbdc7e2d")
    # Users
    # uuid.uuid5(FolioUUID.base_namespace, "users")
    users = uuid.UUID("dfe6e7b1-c0f5-5af1-b5e6-15344192fa79")
    # uuid.uuid5(FolioUUID.base_namespace, "permissions_user")
    permissions_users = uuid.UUID("ee483b0e-efc9-5f8b-a97a-d13a0af85e00")

    # Acquisitions
    # uuid.uuid5(FolioUUID.base_namespace, "orders")
    orders = uuid.UUID("2dc3e4fe-7a32-5014-afb1-099ee08d0626")
    # uuid.uuid5(FolioUUID.base_namespace, "po_lines")
    po_lines = uuid.UUID("c80f7071-83f9-5bc6-8cb6-289f15eb2cd9")
    # uuid.uuid5(FolioUUID.base_namespace, "organizations")
    organizations = uuid.UUID("576a4455-9024-5e46-bf22-b5737de15030")

    # ERM
    # ERM Does not honor generated UUIDs

    # Other
    # uuid.uuid5(FolioUUID.base_namespace, "other")
    other = uuid.UUID("d3a5076d-8bd0-570b-bf84-ee9cb5c4322e")



import uuid
import math
import re

from folio_uuid.folio_namespaces import FOLIONamespaces


class FolioUUID(uuid.UUID):
    """Creates deterministic UUIDs for FOLIO"""

    base_namespace = uuid.UUID("8405ae4d-b315-42e1-918a-d1919900cf3f")

    def __init__(
        self, okapi_url: str, folio_object_type: FOLIONamespaces, legacy_identifier: str
    ):
        """
        Create a deterministic UUID for a FOLIO tenant

        Parameters
        ----------
        okapi_url : str
            The tenant id (or other tenant string)
        folio_object_type : FOLIONamespaces
            Enum helping to avoid collisions within a tenant.
        legacy_identifier : str
            The actual identifier from the legacy system
        """
        clean_id = self.clean_iii_identifiers(legacy_identifier)
        u = uuid.uuid5(folio_object_type.value, f"{okapi_url}:{clean_id}")
        super(FolioUUID, self).__init__(str(u))

    @staticmethod
    def calculate_sierra_check_digit(record_number: int) -> str:
        """Rewritten to python from https://github.com/SydneyUniLibrary/sierra-record-check-digit/blob/master/index.js"""
        m = 2
        x = 0
        i = record_number
        while i > 0:
            a = i % 10
            i = math.floor(i / 10)
            x += a * m
            m += 1
        r = x % 11
        return "x" if r == 10 else str(r)

    @staticmethod
    def clean_iii_identifiers(identifier: str) -> str:
        """
        Cleans potential III/SIERRA/Millennium Identifiers from noise like
        the dot (.) and check digits and potential other things at
        the end of the strings. All other types of identifiers
        should just run through.

        Parameters
        ----------
        identifier : str
            An identifier of any kind.

        """
        reg_pattern = r"^(\.?[bcoiv]?)([0-9xX]{6,8})(@\w{1,5})?$"
        match = re.search(reg_pattern, identifier)
        if not match or not match[1]:  # No match, no III id
            return identifier
        fore = match[1]
        number_part = match[2]
        if len(number_part) > 6:
            # Aft  is reachable by aft = match[3]
            last_char = number_part[-1]
            nums = int(number_part[:-1])
            if FolioUUID.calculate_sierra_check_digit(nums) == last_char:
                return f"{fore.replace('.','')}{nums}"
            else:
                return f"{fore.replace('.','')}{number_part}"
        elif len(number_part) == 6:
            return f"{fore.replace('.','')}{number_part}"
        else:
            return identifier



from distutils.core import setup

setup(
    name="folio_uuid",
    packages=["folio_uuid"],
    version="0.1.0",
    license="MIT",
    long_description="",
    description="A library for generating predictive uuids for FOLIO data migrations",
    author="Theodor Tolstoy",
    author_email="pypi.teddes@tolstoy.se",
    url="https://github.com/FOLIO-FSE/folio_uuid",
    download_url="https://github.com/FOLIO-FSE/folio_uuid/archive/v_01.tar.gz",
    keywords=["FOLIO", "FOLIO_LSP", "OKAPI", "API Wrapper", "UUID"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ],
)
